package com.muyu.cloud.etl.service.impl;

import com.muyu.cloud.etl.service.DataValueService;
import com.muyu.cloud.etl.service.SourceService;
import com.muyu.domain.DataValue;
import com.muyu.domain.Source;
import com.muyu.domain.enums.DataType;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

@Service
@Log4j2
public class DataValueServiceImpl implements DataValueService {
    @Autowired
    private SourceService sourceService;

    @Override
    public List<List<DataValue>> findTableValue(Long basicId, String sql) {
        Source source = sourceService.getInfo(basicId);
        String host = source.getHost();
        String port = source.getPort();
        String databaseName = source.getDatabaseName();
        String databaseType = source.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + source.getConnectionParams();
        String user = source.getUsername();
        String password = source.getPassword();
        List<List<DataValue>> list = new ArrayList<>();
        Connection conn=null;

        try {
            ArrayList<DataValue> dataValues = new ArrayList<>();
            conn = DriverManager.getConnection(url, user, password);
            try {
                PreparedStatement preparedStatement = conn.prepareStatement(sql);
                ResultSet resultSet = preparedStatement.executeQuery();
                ResultSetMetaData metaData = resultSet.getMetaData();
                int columnCount = metaData.getColumnCount();
                while (resultSet.next()){
                    for (int i = 1; i <= columnCount; i++) {

                        String columnTypeName = metaData.getColumnTypeName(i);

                        DatabaseMetaData metaData1 = conn.getMetaData();

                        ResultSet columns = metaData1.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));

                        String remarks =null;

                        while (columns.next()){
                            remarks = columns.getString("REMARKS");
                            log.info("字段备注:"+remarks);
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                .type(DataType.getJavaType(columnTypeName))
                                .build();
                        dataValues.add(build);
                    }
                    list.add(dataValues);
                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }


        return list;
    }

    @Override
    public List<DataValue> findTableValueList(Long basicId, String tableName) {
        Source source = sourceService.getInfo(basicId);
        String host = source.getHost();
        String port = source.getPort();
        String databaseName = source.getDatabaseName();
        String databaseType = source.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + source.getConnectionParams();
        String user = source.getUsername();
        String password = source.getPassword();
        Connection conn=null;
        ArrayList<DataValue> dataValues = new ArrayList<>();
        try {

            conn = DriverManager.getConnection(url, user, password);
            try {
                PreparedStatement preparedStatement = conn.prepareStatement("SELECT  * FROM " + tableName);
                ResultSet resultSet = preparedStatement.executeQuery();
                ResultSetMetaData metaData = resultSet.getMetaData();
                int columnCount = metaData.getColumnCount();
                while (resultSet.next()){
                    for (int i = 1; i <= columnCount; i++) {

                        String columnTypeName = metaData.getColumnTypeName(i);

                        DatabaseMetaData metaData1 = conn.getMetaData();

                        ResultSet columns = metaData1.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));

                        String remarks =null;

                        while (columns.next()){
                            remarks = columns.getString("REMARKS");
                            log.info("字段备注:"+remarks);
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                .type(DataType.getJavaType(columnTypeName))
                                .build();
                        dataValues.add(build);
                    }

                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }


        return dataValues;
    }
}
