package com.muyu.cloud.etl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.cloud.etl.mapper.SourceMapper;
import com.muyu.cloud.etl.service.SourceService;
import com.muyu.cloud.etl.service.StructureService;
import com.muyu.cloud.etl.service.TableInfoService;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.domain.Source;
import com.muyu.domain.Structure;
import com.muyu.domain.TableInfo;
import com.muyu.domain.req.SourceReq;
import groovy.lang.Lazy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import java.sql.*;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Service
public class SourceServiceImpl extends ServiceImpl<SourceMapper, Source> implements SourceService {
    @Autowired
    @Lazy
    private SourceMapper sourceMapper;
    @Autowired
    private TableInfoService tableInfoService;
    @Autowired
    private StructureService structureService;

    @Override
    public List<Source> selectSourceList(SourceReq sourceReq) {
       return sourceMapper.selectSourceList(sourceReq);
    }

    @Override
    public Source getInfo(Long id) {
        LambdaQueryWrapper<Source> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(Source::getId, id);
        return this.sourceMapper.selectOne(lambdaQueryWrapper);
    }

    //新增
    @Override
    public int insertBasicConfigInfo(Source sourceReq) {
        int insert = sourceMapper.insert(sourceReq);
        return insert;
    }

    @Override
    public Integer deleteByIds(String ids) {
        LambdaQueryWrapper<Source> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(Source::getId, ids);
        int delete = sourceMapper.delete(lambdaQueryWrapper);
        return delete;
    }


    @Override
    public int updataSource(Source sourceReq) {
      Integer i=  sourceMapper.updataSource(sourceReq);
        return i;
    }

    @Override
    public int connectionTest(Source source) {
        String host = source.getHost();
        String port = source.getPort();
        String databaseName = source.getDatabaseName();
        String databaseType = source.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + source.getConnectionParams();
        String user = source.getUsername();
        String password = source.getPassword();
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(url, user, password);
            System.out.println("Connected to the MySQL server successfully.");
//            DatabaseMetaData metaData = conn.getMetaData();
//
//            ResultSet rs = metaData.getTables(databaseName,
//                    null, "%", new String[]{"TABLE", "VIEW"});
//
//            while (rs.next()) {
//                String tableName = rs.getString("TABLE_NAME"); // 或者 rs.getString(3)，但使用列名更清晰
//                System.out.println("Table Name: " + tableName);
//            }
//            rs.close();  // 记得关闭ResultSet
//
//            System.out.println(conn);
//            TableInfo tableInfoInsert = TableInfo.builder()
//                    .basicId(source.getId())
//                    .parentId(0L)
//                    .tableRemark("")
//                    .center("Y")
//                    .type("dataSource")
//                    .tableName(source.getDataSourcesSystemName() + "(" + databaseName + ")")
//                    .createBy(SecurityUtils.getUsername())
//                    .createTime(new Date())
//                    .build();
        } catch (SQLException e) {
            return 0;
        }

        return  1;
    }

    @Override
    public int structure(Source source) {

        String host = source.getHost();
        String port = source.getPort();
        String databaseName = source.getDatabaseName();
        String databaseType = source.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + source.getConnectionParams();
        String user = source.getUsername();
        String password = source.getPassword();
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(url, user, password);
            System.out.println("Connected to the MySQL server successfully.");

            TableInfo tableInfoInsert = TableInfo.builder()
                    .basicId(source.getId())
                    .parentId(0L)
                    .tableRemark("")
                    .center("Y")
                    .type("dataSource")
                    .tableName(source.getDataSourcesSystemName() + "(" + databaseName + ")")
                    .createBy(SecurityUtils.getUsername())
                    .createTime(new Date())
                    .build();


            tableInfoService.saveOrUpdate(tableInfoInsert, new LambdaUpdateWrapper<TableInfo>(TableInfo.class) {{
                eq(TableInfo::getTableName, tableInfoInsert.getTableName());
                eq(TableInfo::getBasicId, source.getId());
            }});


            TableInfo tableInfo = tableInfoService.selectTableInfoByName(tableInfoInsert);


            DatabaseMetaData metaData = conn.getMetaData();

            ResultSet rs = metaData.getTables(databaseName,
                    null, "%", new String[]{"TABLE", "VIEW"});

            while (rs.next()) {
                String tableName = rs.getString("TABLE_NAME"); // 或者 rs.getString(3)，但使用列名更清晰
                System.out.println("Table Name: " + tableName);
                String tableRemark = rs.getString("REMARKS");
                Connection finalConn = conn;
                PreparedStatement ps = conn.prepareStatement("select * from " + tableName);
                ResultSet rset = ps.executeQuery();
                Long rowCount = 0L;
                while (rset.next()) {
                    rowCount++;
                }

                TableInfo build = TableInfo.builder()
                        .basicId(source.getId())
                        .tableName(tableName)
                        //bug点，tableRemark为空，造成空指针异常
                        .tableRemark(tableRemark == null ? "" : tableRemark)
                        .parentId(tableInfo.getId())
                        .type("dataTable")
                        .center("Y")
                        .updateBy(SecurityUtils.getUsername())
                        .dataNum(rowCount)
                        .updateTime(new Date())
                        .build();

                tableInfoService.saveOrUpdate(build, new LambdaUpdateWrapper<>(TableInfo.class) {{
                    eq(TableInfo::getTableName, build.getTableName());
                    eq(TableInfo::getBasicId, source.getId());
                }});


                TableInfo table = tableInfoService.selectTableInfoByName(build);

                //线程池
                ExecutorService threadPool = Executors.newCachedThreadPool();

                threadPool.submit(() -> {
                    try {
                        syncData(finalConn, databaseName, table);
                    } catch (SQLException e) {
                        throw new RuntimeException(e);
                    }
                });
                ps.close();
            }
            conn.close();

        } catch (SQLException e) {
            return 0;
        }

        return  1;
    }

    @Override
    public int redis(Source source) {
        JedisPoolConfig jedisPoolConfig = new JedisPoolConfig();
        jedisPoolConfig.setMaxTotal(100);
        jedisPoolConfig.setMaxIdle(100);
        jedisPoolConfig.setMaxWaitMillis(5);

        JedisPool jedisPool = new JedisPool(jedisPoolConfig, "172.13.1.1", 6379, 3000, null);
        Jedis jedis = jedisPool.getResource();
        Set<String> keys = jedis.keys("*");
        for (String key : keys) {
            String value = jedis.get(key);

            String[] split = value.split(",");
            for (String s : split) {
                System.out.println(s);
            }

//            System.out.println(value);
        }

        return 0;
    }


    public void syncData(Connection conn, String databaseName, TableInfo table) throws SQLException {
        ExecutorService threadPool = Executors.newCachedThreadPool();

        PreparedStatement ps = conn.prepareStatement(
                "    SELECT  " +
                        "    COLUMN_NAME , " +
                        "    COLUMN_COMMENT  ," +
                        "    CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END ," +
                        "    CASE \n" +
                        "    WHEN DATA_TYPE = 'int' THEN 'Integer' " +
                        "    WHEN DATA_TYPE = 'bigint' THEN 'Long' " +
                        "    WHEN DATA_TYPE = 'varchar' THEN 'String' " +
                        "    WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal' " +
                        "    WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean'" +
                        "    ELSE DATA_TYPE   \n" +
                        "    END , " +
                        "    DATA_TYPE ,   \n" +
                        "    COLUMN_TYPE ,  \n" +
                        "    CHARACTER_MAXIMUM_LENGTH ,  \n" +
                        "    NUMERIC_SCALE ,  \n" +
                        "    IS_NULLABLE ,  \n" +
                        "    COLUMN_DEFAULT   \n" +
                        "FROM  INFORMATION_SCHEMA.COLUMNS  WHERE  \n" +
                        "TABLE_SCHEMA = '" + databaseName + "'   \n" +
                        "AND TABLE_NAME = '" + table.getTableName() + "'");
//        "SELECT COLUMN_NAME,DATA_TYPE,IS_NULLABLE,COLUMN_KEY,COLUMN_DEFAULT,
//        COLUMN_COMMENT,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,
//        NUMERIC_SCALE FROM INFORMATION_SCHEMA.COLUMNS
//        WHERE TABLE_SCHEMA = '数据库名' AND TABLE_NAME = '表名'"
        ResultSet resultSet = ps.executeQuery();
        while (resultSet.next()) {
            String columnName = String.valueOf(resultSet.getString(1));
            String columnComment = String.valueOf(resultSet.getObject(2));
            String columnKey = String.valueOf(resultSet.getObject(3));
            String end = String.valueOf(resultSet.getObject(4));
            String dataType = String.valueOf(resultSet.getObject(5));
            String columnType = String.valueOf(resultSet.getObject(6));
            String characterMaximumLength = String.valueOf(resultSet.getInt(7));
            String NumericScale = String.valueOf(resultSet.getInt(8));
            String isNullable = String.valueOf(resultSet.getObject(9));
            String columnDefault = String.valueOf(resultSet.getObject(10));

            Structure build = Structure.builder()
                    .tableId(table.getId())
                    .columnName(String.valueOf(columnName))
                    .columnRemark(columnComment)
                    .isPrimary("是".equals(columnKey) ? "Y" : "N")
                    .javaType(end)
                    .columnType(dataType)
                    .columnType(columnType)
                    .columnLength(characterMaximumLength)
                    .columnDecimals(NumericScale)
                    .isNull("YES".equals(isNullable) ? "Y" : "N")
                    .defaultValue(columnDefault)
                    .build();

            threadPool.submit(() -> {
                structureService.saveOrUpdate(build, new LambdaUpdateWrapper<Structure>() {{
                    eq(Structure::getTableId, build.getTableId());
                    eq(Structure::getColumnName, build.getColumnName());
                    eq(Structure::getColumnRemark, build.getColumnRemark());
                }});
            });


//            PreparedStatement preparedStatement = conn.prepareStatement("select columnName from  " + table.getTableName());
//            ResultSet resultSet1 = preparedStatement.executeQuery();
//            while (resultSet1.next()) {
//
//            }

        }
        threadPool.shutdown();
        ps.close();
    }




}
